@@ -1,30 +1,30 @@
 /*
  *   Copyright 2013 Martin Gräßlin <mgraesslin@kde.org>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License as published by the Free Software Foundation; either
+ *   version 2.1 of the License, or (at your option) any later version.
  *
- *   This program is distributed in the hope that it will be useful,
+ *   This library is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *   Lesser General Public License for more details.
  *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with this library.  If not, see <http://www.gnu.org/licenses/>.
  */
 #include "nettesthelper.h"
 #include <netwm.h>
 #include <qtest_widgets.h>
 #include <QProcess>
+// system
+#include <unistd.h>
 
 class Property : public QScopedPointer<xcb_get_property_reply_t, QScopedPointerPodDeleter>
 {
 public:
-    Property(xcb_get_property_reply_t *p = 0) : QScopedPointer<xcb_get_property_reply_t, QScopedPointerPodDeleter>(p) {}
+    Property(xcb_get_property_reply_t *p = nullptr) : QScopedPointer<xcb_get_property_reply_t, QScopedPointerPodDeleter>(p) {}
 };
 
 #define INFO NETWinInfo info(m_connection, m_testWindow, m_rootWindow, NET::WMAllProperties, NET::WM2AllProperties, NET::Client);
@@ -37,7 +37,7 @@ public:
 #define GETPROP(type, length, formatSize) \
     xcb_get_property_cookie_t cookie = xcb_get_property_unchecked(connection(), false, m_testWindow, \
                                        atom, type, 0, length); \
-    Property reply(xcb_get_property_reply(connection(), cookie, Q_NULLPTR)); \
+    Property reply(xcb_get_property_reply(connection(), cookie, nullptr)); \
     QVERIFY(!reply.isNull()); \
     QCOMPARE(reply->format, uint8_t(formatSize)); \
     QCOMPARE(reply->value_len, uint32_t(length));
@@ -45,7 +45,7 @@ public:
 #define VERIFYDELETED(t) \
     xcb_get_property_cookie_t cookieDeleted = xcb_get_property_unchecked(connection(), false, m_testWindow, \
             atom, t, 0, 1); \
-    Property replyDeleted(xcb_get_property_reply(connection(), cookieDeleted, Q_NULLPTR)); \
+    Property replyDeleted(xcb_get_property_reply(connection(), cookieDeleted, nullptr)); \
     QVERIFY(!replyDeleted.isNull()); \
     QVERIFY(replyDeleted->type == XCB_ATOM_NONE);
 
@@ -61,6 +61,7 @@ private Q_SLOTS:
     void testBlockCompositing();
     void testUserTime();
     void testStartupId();
+    void testDesktopFileName();
     void testHandledIcons_data();
     void testHandledIcons();
     void testPid();
@@ -85,9 +86,13 @@ private Q_SLOTS:
     void testInput();
     void testInitialMappingState_data();
     void testInitialMappingState();
+    void testIconPixmap_data();
+    void testIconPixmap();
     void testTransientFor();
     void testProtocols_data();
     void testProtocols();
+    void testOpaqueRegion_data();
+    void testOpaqueRegion();
 
 private:
     void performNameTest(xcb_atom_t atom, const char *(NETWinInfo:: *getter)(void)const, void (NETWinInfo:: *setter)(const char *), NET::Property property);
@@ -97,6 +102,7 @@ private:
         return m_connection;
     }
     xcb_connection_t *m_connection;
+    QVector<xcb_connection_t*> m_connections;
     QScopedPointer<QProcess> m_xvfb;
     xcb_window_t m_rootWindow;
     xcb_window_t m_testWindow;
@@ -105,39 +111,49 @@ private:
 void NetWinInfoTestClient::initTestCase()
 {
     qsrand(QDateTime::currentMSecsSinceEpoch());
-    // first reset just to be sure
-    m_connection = Q_NULLPTR;
-    m_rootWindow = XCB_WINDOW_NONE;
-    m_testWindow = XCB_WINDOW_NONE;
-    // start Xvfb
-    m_xvfb.reset(new QProcess);
-    // randomize the display id in [1, 98]
-    // 0 is not used because it conflicts with "normal" X server
-    // 99 is not used because it's used by KDE's CI infrastructure
-    const QString id = QStringLiteral(":") + QString::number((qrand() % 98) + 1);
-    m_xvfb->start(QStringLiteral("Xvfb"), QStringList() << id);
-    QVERIFY(m_xvfb->waitForStarted());
-    QCOMPARE(m_xvfb->state(), QProcess::Running);
-    // give it some time before we open the X Display
-    QTest::qWait(100);
-    // create X connection
-    int screen = 0;
-    m_connection = xcb_connect(qPrintable(id), &screen);
-    QVERIFY(m_connection);
-    m_rootWindow = KXUtils::rootWindow(m_connection, screen);
 }
 
 void NetWinInfoTestClient::cleanupTestCase()
 {
     // close connection
-    xcb_disconnect(connection());
-    // kill Xvfb
-    m_xvfb->terminate();
-    m_xvfb->waitForFinished();
+    while (!m_connections.isEmpty()) {
+        xcb_disconnect(m_connections.takeFirst());
+    }
 }
 
 void NetWinInfoTestClient::init()
 {
+    // first reset just to be sure
+    m_connection = nullptr;
+    m_rootWindow = XCB_WINDOW_NONE;
+    m_testWindow = XCB_WINDOW_NONE;
+    // start Xvfb
+    m_xvfb.reset(new QProcess);
+    // use pipe to pass fd to Xvfb to get back the display id
+    int pipeFds[2];
+    QVERIFY(pipe(pipeFds) == 0);
+    m_xvfb->start(QStringLiteral("Xvfb"), QStringList{ QStringLiteral("-displayfd"), QString::number(pipeFds[1]) });
+    QVERIFY(m_xvfb->waitForStarted());
+    QCOMPARE(m_xvfb->state(), QProcess::Running);
+
+    // reads from pipe, closes write side
+    close(pipeFds[1]);
+
+    QFile readPipe;
+    QVERIFY(readPipe.open(pipeFds[0], QIODevice::ReadOnly, QFileDevice::AutoCloseHandle));
+    QByteArray displayNumber = readPipe.readLine();
+    readPipe.close();
+
+    displayNumber.prepend(QByteArray(":"));
+    displayNumber.remove(displayNumber.size() -1, 1);
+
+    // create X connection
+    int screen = 0;
+    m_connection = xcb_connect(displayNumber.constData(), &screen);
+    QVERIFY(m_connection);
+    QVERIFY(!xcb_connection_has_error(m_connection));
+    m_rootWindow = KXUtils::rootWindow(m_connection, screen);
+
     // create test window
     m_testWindow = xcb_generate_id(m_connection);
     uint32_t values[] = {XCB_EVENT_MASK_PROPERTY_CHANGE};
@@ -155,6 +171,12 @@ void NetWinInfoTestClient::cleanup()
     xcb_unmap_window(m_connection, m_testWindow);
     xcb_destroy_window(m_connection, m_testWindow);
     m_testWindow = XCB_WINDOW_NONE;
+
+    // delay till clenupTestCase as otherwise xcb reuses the same memory address
+    m_connections << connection();
+    // kill Xvfb
+    m_xvfb->terminate();
+    m_xvfb->waitForFinished();
 }
 
 void NetWinInfoTestClient::waitForPropertyChange(NETWinInfo *info, xcb_atom_t atom, NET::Property prop, NET::Property2 prop2)
@@ -264,6 +286,28 @@ void NetWinInfoTestClient::testStartupId()
     QCOMPARE(info.startupId(), "foo");
 }
 
+void NetWinInfoTestClient::testDesktopFileName()
+{
+    QVERIFY(connection());
+    ATOM(_KDE_NET_WM_DESKTOP_FILE)
+    UTF8
+    INFO
+
+    QVERIFY(!info.desktopFileName());
+    info.setDesktopFileName("foo");
+    QCOMPARE(info.desktopFileName(), "foo");
+
+    // compare with the X property
+    QVERIFY(atom != XCB_ATOM_NONE);
+    QVERIFY(utf8String != XCB_ATOM_NONE);
+    GETPROP(utf8String, 3, 8)
+    QCOMPARE(reinterpret_cast<const char *>(xcb_get_property_value(reply.data())), "foo");
+
+    // and wait for our event
+    waitForPropertyChange(&info, atom, NET::Property(0), NET::WM2DesktopFileName);
+    QCOMPARE(info.desktopFileName(), "foo");
+}
+
 void NetWinInfoTestClient::testHandledIcons_data()
 {
     QTest::addColumn<bool>("handled");
@@ -811,6 +855,52 @@ void NetWinInfoTestClient::testInitialMappingState()
     QTEST(info.initialMappingState(), "expected");
 }
 
+void NetWinInfoTestClient::testIconPixmap_data()
+{
+    QTest::addColumn<quint32>("flags");
+    QTest::addColumn<quint32>("icon");
+    QTest::addColumn<quint32>("mask");
+    QTest::addColumn<quint32>("expectedPixmap");
+    QTest::addColumn<quint32>("expectedMask");
+
+    QTest::newRow("invalid-flags") << 1u << 2u << 3u << 0u << 0u;
+    QTest::newRow("pixmap-flags") << 4u << 2u << 3u << 2u << 0u;
+    QTest::newRow("mask-flags") << 32u << 2u << 3u << 0u << 3u;
+    QTest::newRow("pixmap-mask-flags") << 36u << 2u << 3u << 2u << 3u;
+}
+
+void NetWinInfoTestClient::testIconPixmap()
+{
+    QVERIFY(connection());
+    INFO
+
+    QCOMPARE(info.icccmIconPixmap(), 0u);
+    QCOMPARE(info.icccmIconPixmapMask(), 0u);
+    QFETCH(quint32, flags);
+    QFETCH(quint32, icon);
+    QFETCH(quint32, mask);
+
+    // icon pixmap needs to be changed through wm hints
+    uint32_t values[] = {
+        flags,
+        1, /* Input */
+        XCB_NONE, /* Normal State */
+        icon,     /* icon pixmap */
+        XCB_NONE, /* icon window */
+        XCB_NONE, /* icon x */
+        XCB_NONE, /* icon y */
+        mask,     /* icon mask */
+        XCB_NONE  /* group leader */
+    };
+    xcb_change_property(connection(), XCB_PROP_MODE_REPLACE, m_testWindow,
+                        XCB_ATOM_WM_HINTS, XCB_ATOM_WM_HINTS, 32, 9, values);
+    xcb_flush(connection());
+    // only updated after event
+    waitForPropertyChange(&info, XCB_ATOM_WM_HINTS, NET::Property(0), NET::WM2IconPixmap);
+    QTEST(info.icccmIconPixmap(), "expectedPixmap");
+    QTEST(info.icccmIconPixmapMask(), "expectedMask");
+}
+
 void NetWinInfoTestClient::testTransientFor()
 {
     QVERIFY(connection());
@@ -887,7 +977,7 @@ void NetWinInfoTestClient::testActivities()
     QFETCH(QByteArray, activities);
 
     // activities needs to be changed using xcb
-    info.setActivities(activities.isNull() ? Q_NULLPTR : activities.constData());
+    info.setActivities(activities.isNull() ? nullptr : activities.constData());
     xcb_flush(connection());
 
     // only updated after event
@@ -1016,6 +1106,61 @@ void NetWinInfoTestClient::testProtocols()
     QCOMPARE(info.protocols(), NET::Protocols(NET::NoProtocol));
 }
 
+void NetWinInfoTestClient::testOpaqueRegion_data()
+{
+    QTest::addColumn<QVector<QRect> >("geometries");
+
+    QTest::newRow("none") << QVector<QRect>();
+    QTest::newRow("empty") << QVector<QRect>({QRect(0, 0, 0, 0)});
+    QTest::newRow("one rect") << QVector<QRect>({QRect(10, 20, 30, 40)});
+    QTest::newRow("two rect") << QVector<QRect>({QRect(10, 20, 30, 40), QRect(1, 2, 4, 5)});
+    QTest::newRow("multiple") << QVector<QRect>({QRect(10, 20, 30, 40),
+                                                 QRect(1, 2, 4, 5),
+                                                 QRect(100, 0, 200, 400),
+                                                 QRect(1, 2, 4, 5)});
+}
+
+void NetWinInfoTestClient::testOpaqueRegion()
+{
+    QVERIFY(connection());
+    ATOM(_NET_WM_OPAQUE_REGION)
+    INFO
+
+    QCOMPARE(info.opaqueRegion().size(), std::size_t(0));
+
+    QFETCH(QVector<QRect>, geometries);
+    QVector<qint32> data;
+    for (auto it = geometries.constBegin(); it != geometries.constEnd(); ++it) {
+        const QRect &r = *it;
+        data << r.x();
+        data << r.y();
+        data << r.width();
+        data << r.height();
+    }
+
+    xcb_change_property(connection(), XCB_PROP_MODE_REPLACE, m_testWindow, atom, XCB_ATOM_CARDINAL, 32, data.size(), data.constData());
+    xcb_flush(connection());
+
+    // only updated after event
+    waitForPropertyChange(&info, atom, NET::Property(0), NET::WM2OpaqueRegion);
+    const auto opaqueRegion = info.opaqueRegion();
+    QCOMPARE(opaqueRegion.size(), std::size_t(geometries.size()));
+
+    for (std::size_t i = 0; i < opaqueRegion.size(); ++i) {
+        auto r1 = opaqueRegion.at(i);
+        auto r2 = geometries.at(i);
+        QCOMPARE(r1.pos.x, r2.x());
+        QCOMPARE(r1.pos.y, r2.y());
+        QCOMPARE(r1.size.width, r2.width());
+        QCOMPARE(r1.size.height, r2.height());
+    }
+
+    xcb_delete_property(connection(), m_testWindow, atom);
+    xcb_flush(connection());
+    waitForPropertyChange(&info, atom, NET::Property(0), NET::WM2OpaqueRegion);
+    QCOMPARE(info.opaqueRegion().size(), std::size_t(0));
+}
+
 QTEST_GUILESS_MAIN(NetWinInfoTestClient)
 
 #include "netwininfotestclient.moc"
