@@ -87,7 +87,7 @@ static QByteArray get_cstr(const QString &item_P);
 static QStringList get_fields(const QString &txt_P);
 static QString escape_str(const QString &str_P);
 
-class KStartupInfo::Data
+class Q_DECL_HIDDEN KStartupInfo::Data
     : public KStartupInfoData
 {
 public:
@@ -97,7 +97,8 @@ public:
     unsigned int age;
 };
 
-struct KStartupInfoId::Private {
+struct Q_DECL_HIDDEN KStartupInfoId::Private
+{
     Private() : id("") {}
 
     QString to_text() const;
@@ -105,7 +106,8 @@ struct KStartupInfoId::Private {
     QByteArray id; // id
 };
 
-struct KStartupInfoData::Private {
+struct Q_DECL_HIDDEN KStartupInfoData::Private
+{
     Private() : desktop(0), wmclass(""), hostname(""),
         silent(KStartupInfoData::Unknown), screen(-1), xinerama(-1), launched_by(0) {}
 
@@ -127,7 +129,7 @@ struct KStartupInfoData::Private {
     QString application_id;
 };
 
-class KStartupInfo::Private
+class Q_DECL_HIDDEN KStartupInfo::Private
 {
 public:
     // private slots
@@ -964,11 +966,11 @@ QByteArray KStartupInfo::windowStartupId(WId w_P)
     if (!QX11Info::isPlatformX11()) {
         return QByteArray();
     }
-    NETWinInfo info(QX11Info::connection(), w_P, QX11Info::appRootWindow(), nullptr, NET::WM2StartupId | NET::WM2GroupLeader);
+    NETWinInfo info(QX11Info::connection(), w_P, QX11Info::appRootWindow(), NET::Properties(), NET::WM2StartupId | NET::WM2GroupLeader);
     QByteArray ret = info.startupId();
     if (ret.isEmpty() && info.groupLeader() != XCB_WINDOW_NONE) {
         // retry with window group leader, as the spec says
-        NETWinInfo groupLeaderInfo(QX11Info::connection(), info.groupLeader(), QX11Info::appRootWindow(), nullptr, NET::WM2StartupId);
+        NETWinInfo groupLeaderInfo(QX11Info::connection(), info.groupLeader(), QX11Info::appRootWindow(), NET::Properties(), NET::Properties2());
         ret = groupLeaderInfo.startupId();
     }
     return ret;
@@ -987,7 +989,7 @@ void KStartupInfo::setWindowStartupId(WId w_P, const QByteArray &id_P)
     if (id_P.isNull()) {
         return;
     }
-    NETWinInfo info(QX11Info::connection(), w_P, QX11Info::appRootWindow(), nullptr, nullptr);
+    NETWinInfo info(QX11Info::connection(), w_P, QX11Info::appRootWindow(), NET::Properties(), NET::Properties2());
     info.setStartupId(id_P.constData());
 #else
     Q_UNUSED(w_P)
